import string
import random
import datetime

from django.db import models
from django.contrib.auth.models import User


def _random_block():
    r = random.SystemRandom()
    s = string.lowercase+string.uppercase+string.digits
    return ''.join(r.choice(s) for i in range(64))


class Consumer(models.Model):
    name	= models.CharField(max_length=64)
    key		= models.CharField(max_length=64, default=_random_block, unique=True)
    secret	= models.CharField(max_length=64, default=_random_block)

    def __unicode__(self):
        return self.name


class Credential(models.Model):
    token	= models.CharField(max_length=64, default=_random_block, unique=True)
    secret	= models.CharField(max_length=64, default=_random_block)
    consumer	= models.ForeignKey(Consumer)
    user	= models.ForeignKey(User, blank=True, null=True)
    vericode	= models.CharField(max_length=64, blank=True, default='')
    issued_on	= models.DateTimeField(default=datetime.datetime.now)

    def __unicode__(self):
        return self.token

    def is_token(self):
        # If it's been assigned a user, and it's not awaiting verification, it's a token
        return bool(self.user) and not len(self.vericode)
    is_token.short_description = 'Token?'
    is_token.boolean = True

    def is_temporary(self):
        # If it's not a token, it's temporary
        return not self.is_token()
    is_temporary.short_description = 'Temporary?'
    is_temporary.boolean = True

    def is_authorized(self):
        # If it's been assigned a user, it's been authorized
        return bool(self.user)
    is_authorized.short_description = 'Authorized?'
    is_authorized.boolean = True

    def authorize(self, user):
        self.user	= user
        self.vericode	= _random_block()

    def tokenize(self):
        self.token	= _random_block()
        self.secret	= _random_block()
        self.vericode	= ''


class ReplayBar(models.Model):
    class Meta:
        unique_together = (('token', 'timestamp', 'nonce'),)

    token	= models.ForeignKey(Credential, blank=True, null=True)
    timestamp	= models.DateTimeField()
    nonce	= models.CharField(max_length=64)

    def __unicode__(self):
        return unicode(self.timestamp)
