import	urllib

from	django.http				import HttpResponse
from	django.views.decorators.csrf		import csrf_exempt
from	django.contrib.auth.models		import User

from	sharedapps.turbo_oauth			import models
from	sharedapps.turbo_oauth.oauth_core	import Http400, Http401, oauth_user_required, oauth_only_consumer_required


@csrf_exempt
@oauth_only_consumer_required
def request_token(request):
    # The oauth_callback parameter is required - it must be oob (case sensitive) for this implementation
    if 'oauth_callback' not in request.oauth_data.all_fields: raise Http400('Missing parameter')
    if request.oauth_data.all_fields['oauth_callback'] != 'oob': raise Http400('Non-oob callback')

    # Create unauthorized credential
    # (There's an infinitesimal chance this will fail, and throw an exception)
    credential = models.Credential(consumer=request.oauth_data.consumer)
    credential.save()

    # Return token, secret, cb_confirmed (as 2.1)
    d = {'oauth_token':credential.token,'oauth_token_secret':credential.secret,'oauth_callback_confirmed':'true'}
    r = HttpResponse(urllib.urlencode(d))
    r['Content-Type'] = 'application/x-www-form-urlencoded'
    return r


@csrf_exempt
@oauth_only_consumer_required
def access_token(request):
    # Authorized non-token credential required
    if not request.oauth_data.credential: raise Http400('Missing credential')
    if not request.oauth_data.credential.is_authorized(): raise Http401('Unauthorized credential')
    if request.oauth_data.credential.is_token(): raise Http400('Not a temporary credential')

    # The oauth_verifier parameter is required - it must match the credential
    if 'oauth_verifier' not in request.oauth_data.all_fields: raise Http400('Missing parameter')
    if request.oauth_data.all_fields['oauth_verifier'] != request.oauth_data.credential.vericode:
        raise Http401('Bad verification code')

    # Swap out token and secret, pull vericode 
    # (There's an infinitesimal chance this will fail, and throw an exception)
    request.oauth_data.credential.tokenize()
    request.oauth_data.credential.save()

    # Return token, secret (as 2.3)
    d = {'oauth_token':request.oauth_data.credential.token,'oauth_token_secret':request.oauth_data.credential.secret}
    r = HttpResponse(urllib.urlencode(d))
    r['Content-Type'] = 'application/x-www-form-urlencoded'
    return r


@csrf_exempt
@oauth_only_consumer_required
def turborize(request):
    # Unauthorized credential required
    if not request.oauth_data.credential: raise Http400('Missing credential')
    if request.oauth_data.credential.is_authorized(): raise Http400('Credential already authorized')

    # Create new user
    user = User(username=request.oauth_data.credential.token[:20])
    user.set_unusable_password()
    user.save()
    user.username = 'oauth_user_%d'%user.pk
    user.save()

    # Assign user and vericode
    request.oauth_data.credential.authorize(user)
    request.oauth_data.credential.save()

    # return token, verifier (sort of as 2.2)
    d = {'oauth_token':request.oauth_data.credential.token,'oauth_verifier':request.oauth_data.credential.vericode}
    r = HttpResponse(urllib.urlencode(d))
    r['Content-Type'] = 'application/x-www-form-urlencoded'
    return r
